
#####################################################################
#
# ESPREM Client Init
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

__version__ = "0.7.181100"

def test7( ) :
    return( 7 )

def get_version( ) :
    return __version__ 

    



#####################################################################
#
# ESPREM Client Cache
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import os
import json
import hashlib
import time

from . import s_log , s_config , s_misc

cache_dirpath = "/tmp/pyespremclient_cache2/"

#####################################################################


def is_enabled( ) :

    if( s_config.get_key( "pyespremclient_cache" ) == "on" ) :
        return True

    return False


def hashtopath( ha ) :
    subdirs=ha[0:4]+"/"+ha[4:8]+"/"
    p = cache_dirpath + subdirs + ha + ".json"
    #s_log.write_msg( p )
    return p


def get( request_inputs ) :

    if( not is_enabled( ) ) : return None

    if( request_inputs == None ) : return { }

    inputs_str = json.dumps( request_inputs )
    cache_hash = hashlib.sha256( inputs_str.encode( "utf-8" ) ).hexdigest( )

    cache_filepath = hashtopath( cache_hash ) 
    #s_log.write_msg( "cache_filepath=" + cache_filepath )
    if( not os.path.isfile( cache_filepath ) ) : return None
    with open( cache_filepath , "r" ) as f :
        return( json.load( f ) )


def available( request_inputs ) :

    if( not is_enabled( ) ) : return False

    inputs_str = json.dumps( request_inputs )
    cache_hash = hashlib.sha256( inputs_str.encode( "utf-8" ) ).hexdigest( )
    cache_filepath = cache_dirpath + cache_hash + ".json"
    s_log.write_msg( "cache_filepath=" + cache_filepath )

    return os.path.isfile( cache_filepath ) 


    #return False

    #request_str = request_list[ 0 ] + json.dumps( request_list[ 1 ] )
    #cache_hash = hashlib.sha256( request_str.encode('utf-8') ).hexdigest( )
    #cache_filepath = cache_dirpath + cache_hash + ".json"
    #s_log.write_msg("cache_filepath="+cache_filepath)

    #return os.path.isfile( cache_filepath ) 

# The response
def set( request_inputs , request_response ) :
    if( not is_enabled( ) ) : return False
    # Create/Recreate cache directory
    inputs_str = json.dumps( request_inputs )
    cache_hash = hashlib.sha256( inputs_str.encode( "utf-8" ) ).hexdigest( )
    cache_filepath = hashtopath( cache_hash ) 
    if( os.path.isfile( cache_filepath ) ) : return None
    init_path( cache_filepath )
    response = request_response.copy( )
    #response[ "_pyesprem_cache_time_" ] = int(time.time())
    with open( cache_filepath , "w" ) as f :
        #f.write( json.dumps( response ) )
        f.write( s_misc.json_encode(response) )
    s_log.write_msg( "cache updated" )

def get_data( request_list ) :
    if( not is_enabled( ) ) : return False
    request_str = request_list[ 0 ] + json.dumps( request_list[ 1 ] )
    cache_hash = hashlib.sha256( request_str.encode('utf-8') ).hexdigest( )
    cache_filepath = cache_dirpath + cache_hash + ".json"
    with open( cache_filepath , "r" ) as f :
        return( json.load( f ) )


def init_path( fp ) :
    #s_log.write_msg(fp)
    #s_log.write_msg(os.path.dirname(fp))

    fpdirname=os.path.dirname(fp)

    if( not os.path.isdir( fpdirname ) ) :
        os.makedirs( fpdirname , exist_ok = True )

#####################################################################

#init_directory( )
    



#####################################################################
#
# ESPREM Client 
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import os
import re
import io
import json
import atexit
#from pathlib import Path

#####################################################################

from . import s_log , s_config , s_net , s_cache , s_misc 

####################################################################

s_log.write_msg( "    ####    ####    ####    ####" )

####################################################################

if( "NOM_SERVER_CONFIGURATION" in os.environ ) :
    s_log.write_msg( "Found NOM_SERVER_CONFIGURATION" )
    s_config.init_fromfile( os.environ[ "NOM_SERVER_CONFIGURATION" ] , "sparc_esprem" )
    
####################################################################

def log_msg( msg ) :
    s_log.write_msg( msg )

def version( ) :
    return( config_get_key( "_version_" ) )

def config_get( ) :
    return( s_config.get_config( ) )

def config_get_key( config_key ) :
    return( s_config.get_key( config_key ) )

def config_set_key( config_key , config_val ) :
    s_config.set_key( config_key , config_val )

def config_init_fromfile( config_path , config_key ) :
    return s_config.init_fromfile( config_path , config_key )

def json_encode( data ) :
    return s_misc.json_encode( data )

def pickle_load( fn ) :
    return s_misc.pickle_load( fn )

def cache_available( request_inputs ) :
    return s_cache.available( request_inputs )

def cache_set( request_inputs , request_response ) :
    s_cache.set( request_inputs , request_response )

def cache_get( request_inputs ) :
    return s_cache.get( request_inputs )

####################################################################

def do_request( request_endpoint , request_params , request_id = 0 , request_timeout_in = 60 ) :

    #if( s_cache.available( [ request_endpoint , request_params ] ) ) :
    #    s_log.write_msg( "CACHE HIT" )
    #    return s_cache.get_data( [ request_endpoint , request_params ] )
    #s_log.write_msg( "CACHE MISS" )
    
    ####################################################################

    #if( s_cache.available( request_params) ) :
    #    log_msg( "CACHE AVAILABLE" )

    request_timeout = int( s_config.get_key( "pyespremclient_timeout" , 600 ) )
    log_msg( "request_timeout=" + str( request_timeout )  )

    ####################################################################

    response = s_net.get_response( request_endpoint , request_params , request_id , request_timeout ) 

    if( "code" in response ) :
        res_str = json.dumps( response , indent = 4 )
        s_log.write_msg( res_str )
        return( response )
        #assert False , "code"

    if( "_error" in response ) :
        res_str = json.dumps( response , indent = 4 )
        s_log.write_msg( res_str )
        return( response )
        #assert False , "_error"

    ####################################################################

    #s_cache.update( request_params , response )

    ####################################################################

    return( response )







#####################################################################
#
# ESPREM Client Net
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import json
import time
import requests
from requests import ReadTimeout, ConnectTimeout, HTTPError, Timeout 

from . import s_log , s_config

#####################################################################

def get_response( request_endpoint , request_params , request_id , request_timeout ) :

    s_log.write_msg( "s_net.get_response START" )

    ################################################################

    headers =   {
                    "content-type" : "application/json" 
                }
    
    payload =   {
                    "method" : request_endpoint ,
                    "params" : request_params ,
                    "jsonrpc" : "2.0" ,
                    "id" : request_id ,
                }

    ################################################################

    try :

        response = requests.post( s_config.get_key( "url_endpoint" ) , json = payload , headers = headers , timeout = request_timeout )


        #s_log.write_msg( "s_net.get_response SLEEP 5" )
        #time.sleep(5)

        s_log.write_msg( "s_net.get_response END" )

        if( response.status_code != 200 ) :

            response =  {
                            "_error" :  {
                                            "error" : True ,
                                            "error_code" : "status_code" ,
                                            "error_msg" : response.text ,
                                        }
                        }    

            return( response )    

    except requests.ConnectionError as e :

        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "ConnectionError" ,
                                        "error_msg" : str( e ) ,
                                    }
                    }    

        return( response )    

    except requests.HTTPError as e :

        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "HTTPError" ,
                                        "error_msg" : str( e ) ,
                                    }
                    }    

        return( response )    

    except requests.exceptions.InvalidSchema as e :

        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "InvalidSchema" ,
                                        "error_msg" : str( e ) ,
                                    }
        
                    }    

        return( response )   

    except requests.exceptions.InvalidURL as e :

        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "InvalidURL " ,
                                        "error_msg" : str(e) ,
                                    }
                    }    

        return( response )    

    except requests.exceptions.MissingSchema as e :

        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "MissingSchema " ,
                                        "error_msg" : str( e ) ,
                                    }
                    }    

        return( response )    

    except( ConnectTimeout, HTTPError, ReadTimeout, Timeout ) :
        response =  {
                        "_error" :  {
                                        "error" : True ,
                                        "error_code" : "TIMEOUT" ,
                                        "error_msg" : "TIMEOUT" ,
                                    }
                    }    

        return( response )  


    ################################################################

    response = response.json( )

    if( "result" in response ) :
        return( response[ "result" ] )
    
    ################################################################

    return( response[ "error" ] )


    

    



#####################################################################
#
# ESPREM Client Log
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import json

from datetime import datetime

datetime.now( ).strftime("%Y_%m_%d-%I_%M_%S_%p")

#####################################################################

def default_json( t ) :
    return f'{t}'

def write_msg( msg_raw ) :

    msg = msg_raw

    if(msg==None): msg="!NONE!"
    
    if( isinstance( msg , list ) ) :
        msg = ",".join( map( str , msg ) )
    
    if( isinstance( msg , dict ) ) :
        msg = json.dumps( msg , default = default_json )


    if( isinstance( msg , ( int , float , bool ) ) ) :
        msg = str( msg )

    msg = datetime.now( ).strftime( "%Y_%m_%d-%I_%M_%S_%p" ) + " " + msg
    with open( "/tmp/pyespremclient.log" , "a+" , 1 ) as log_file :
        log_file.write( msg + "\n" )


    



#####################################################################
#
# ESPREM Client Misc
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import os
import atexit
import json
import pickle
import numpy

from . import s_log  

#####################################################################

def cleanup( ) :
    s_log.write_msg( "CLEANUP" )

atexit.register( cleanup )
s_log.write_msg( "Registered cleanup" )

#####################################################################

class NumpyEncoder(json.JSONEncoder):
    """ Special json encoder for numpy types """
    def default(self, obj):
        if isinstance(obj, numpy.integer):
            return int(obj)
        elif isinstance(obj, numpy.floating):
            return float(obj)
        elif isinstance(obj, numpy.ndarray):
            return obj.tolist()
        return json.JSONEncoder.default(self, obj)

def json_encode( data ) :
    return json.dumps(data, cls=NumpyEncoder)

#####################################################################

def pickle_load( fn ) :
    if( os.path.isfile( fn ) ) :
        s_log.write_msg( "FOUND " + fn )
        with open( fn , "rb" ) as f :
            return( pickle.load( f ) )
    else:
        return None
        
def pickle_save( data ) :
    #f = open("local_espremrunresponse.pkl","wb")
    #pickle.dump(run_response,f)
    #f.close()
    pass


    



#####################################################################
#
# ESPREM Client Config
#
# Project   : PYESPREMCLIENT
# Author(s) : Zafar Iqbal < zaf@sparc.gr >
# Copyright : (C) 2021 SPARC PC < https://sparc.space/ >
#
# All rights reserved. No warranty, explicit or implicit, provided.
# SPARC PC is and remains the owner of all titles, rights
# and interests in the Software.
#
#####################################################################

import os
import json

#####################################################################

from . import get_version
from . import s_log  

#####################################################################

config = { }

#####################################################################

config[ "_version_" ] = get_version( )

#####################################################################

def get_config( ) :
    return config

def set_key( config_key , config_val ) :
    global config
    s_log.write_msg( "CONFIG SET_KEY " + config_key + " = " + str( config_val ) )
    config[ config_key ] = config_val

def get_key( config_key , default_val = None ) :
    #s_log.write_msg("GET KEY "+config_key)
    if( config_key in config ) :
        #s_log.write_msg("GET KEY FOUND")
        if config[ config_key ] != None :
            return config[ config_key ]
            
    #s_log.write_msg("GET KEY NULL")
    return default_val

def init_fromfile( config_path , config_key ) :

    global config

    #s_log.write_msg(os.getcwd()+","+config_path)
    #s_log.write_msg(os.getcwd())

    try :

        with open( config_path ) as f :
            config_all = json.load( f )
    
    except IOError :

        s_log.write_msg( "ERROR IOError " + config_path )
        return False

    ####################################################################

    if config_key in config_all :
        config.update( config_all[ config_key ] )
    else :
        s_log.write_msg( "ERROR config_key " + config_key )
        return False

    ####################################################################

    s_log.write_msg( "LOADED " + config_path )

    #s_log.write_msg( config )

    if( not "url_endpoint" in config ) :
        s_log.write_msg( "ERROR url_endpoint not found in config" )
        return False

    #s_log.write_msg( config )

    return True

